@@ -177,11 +177,36 @@ class Parser:
         TokenType.EXCEPT,
     }
 
+    JOIN_SIDES = {
+        TokenType.LEFT,
+        TokenType.RIGHT,
+        TokenType.FULL,
+    }
+
+    JOIN_KINDS = {
+        TokenType.INNER,
+        TokenType.OUTER,
+        TokenType.CROSS,
+    }
+
+    BODY_EXPRESSIONS = (
+        exp.Select,
+        exp.From,
+        exp.Join,
+        exp.Where,
+        exp.Group,
+        exp.Having,
+        exp.Order,
+        exp.Union,
+        exp.CTE,
+        exp.Values,
+    )
+
     __slots__ = (
         "functions",
         "error_level",
         "error_message_context",
-        "code",
+        "sql",
         "errors",
         "index_offset",
         "strict_cast",
@@ -209,7 +234,7 @@ class Parser:
         self.reset()
 
     def reset(self):
-        self.code = ""
+        self.sql = ""
         self.errors = []
         self._tokens = []
         self._chunks = [[]]
@@ -218,20 +243,49 @@ class Parser:
         self._next = None
         self._prev = None
 
-    def parse(self, raw_tokens, code=None):
+    def parse(self, raw_tokens, sql=None):
         """
         Parses the given list of tokens and returns a list of syntax trees, one tree
         per parsed SQL statement.
 
         Args
             raw_tokens (list): the list of tokens (:class:`~sqlglot.tokens.Token`).
-            code (str): the original SQL string. Used to produce helpful debug messages.
+            sql (str): the original SQL string. Used to produce helpful debug messages.
 
         Returns
             the list of syntax trees (:class:`~sqlglot.expressions.Expression`).
         """
+        return self._parse(
+            parse_method=self._parse_statement, raw_tokens=raw_tokens, sql=sql
+        )
+
+    def parse_into(self, expression_type, raw_tokens, sql=None):
+        methods = {
+            exp.From: self._parse_from,
+            exp.Group: self._parse_group,
+            exp.Lateral: self._parse_lateral,
+            exp.Join: self._parse_join,
+            exp.Order: self._parse_order,
+            exp.Limit: self._parse_limit,
+            exp.Offset: self._parse_offset,
+            exp.TableAlias: self._parse_table_alias,
+            exp.Table: self._parse_table,
+            exp.CONJUNCTION: self._parse_conjunction,
+        }
+        if expression_type not in methods:
+            raise TypeError(f"No parser registered for {expression_type}")
+
+        method = methods[expression_type]
+        expressions = self._parse(method, raw_tokens, sql)
+
+        if not expressions:
+            raise ValueError(f"Failed to parse into {expression_type}")
+
+        return expressions
+
+    def _parse(self, parse_method, raw_tokens, sql=None):
         self.reset()
-        self.code = code or ""
+        self.sql = sql or ""
         total = len(raw_tokens)
 
         for i, token in enumerate(raw_tokens):
@@ -247,13 +301,12 @@ class Parser:
             self._index = -1
             self._tokens = tokens
             self._advance()
-            expressions.append(self._parse_statement())
+            expressions.append(parse_method())
 
             if self._index < len(self._tokens):
                 self.raise_error("Invalid expression / Unexpected token")
 
             self.check_errors()
-            self.set_parents(expressions)
 
         return expressions
 
@@ -264,22 +317,13 @@ class Parser:
             if self.error_level == ErrorLevel.WARN:
                 logger.error(error)
 
-    def set_parents(self, expressions):
-        for expression in expressions:
-            if not expression:
-                continue
-            for node, parent, key in expression.walk():
-                if isinstance(node, exp.Expression) and parent:
-                    node.parent = parent
-                    node.arg_key = key
-
     def raise_error(self, message, token=None):
         token = token or self._curr or self._prev or Token.string("")
-        start = self._find_token(token, self.code)
+        start = self._find_token(token, self.sql)
         end = start + len(token.text)
-        start_context = self.code[max(start - self.error_message_context, 0) : start]
-        highlight = self.code[start:end]
-        end_context = self.code[end : end + self.error_message_context]
+        start_context = self.sql[max(start - self.error_message_context, 0) : start]
+        highlight = self.sql[start:end]
+        end_context = self.sql[end : end + self.error_message_context]
         self.errors.append(
             ParseError(
                 f"{message}. Line {token.line}, Col: {token.col}.\n"
@@ -308,13 +352,13 @@ class Parser:
                     f"Required keyword: '{k}' missing for {expression.__class__}"
                 )
 
-    def _find_token(self, token, code):
+    def _find_token(self, token, sql):
         line = 1
         col = 1
         index = 0
 
         while line < token.line or col < token.col:
-            if code[index] == "\n":
+            if sql[index] == "\n":
                 line += 1
                 col = 1
             else:
@@ -366,7 +410,9 @@ class Parser:
                 expression=self._parse_string(),
             )
 
-        return self._parse_set_operations(self._parse_expression()) or self._parse_cte()
+        return (
+            self._parse_set_operations(self._parse_expression()) or self._parse_with()
+        )
 
     def _parse_drop(self):
         if self._match(TokenType.TABLE):
@@ -379,7 +425,7 @@ class Parser:
         return self.expression(
             exp.Drop,
             exists=self._parse_exists(),
-            this=self._parse_table(None, schema=True),
+            this=self._parse_table(schema=True),
             kind=kind,
         )
 
@@ -400,7 +446,7 @@ class Parser:
             self.raise_error("Expected TABLE or View")
 
         exists = self._parse_exists(not_=True)
-        this = self._parse_table(alias=None, schema=True)
+        this = self._parse_table(schema=True)
         expression = None
         properties = None
 
@@ -410,7 +456,7 @@ class Parser:
             )
 
         if self._match(TokenType.ALIAS):
-            expression = self._parse_cte()
+            expression = self._parse_with()
 
         options = {
             "engine": None,
@@ -475,9 +521,10 @@ class Parser:
                     for expression in schema.args["expressions"]
                     if expression.this.text("this").upper() in columns
                 ]
-                schema.args["expressions"] = [
-                    e for e in schema.args["expressions"] if e not in partitions
-                ]
+                schema.set(
+                    "expressions",
+                    [e for e in schema.args["expressions"] if e not in partitions],
+                )
                 value = self.expression(exp.Schema, expressions=partitions)
         else:
             value = self._parse_string()
@@ -538,7 +585,7 @@ class Parser:
 
         return self.expression(
             exp.Insert,
-            this=self._parse_table(alias=None, schema=True),
+            this=self._parse_table(schema=True),
             exists=self._parse_exists(),
             partition=self._parse_partition(),
             expression=self._parse_select(),
@@ -550,7 +597,7 @@ class Parser:
 
         return self.expression(
             exp.Delete,
-            this=self._parse_table(alias=None, schema=True),
+            this=self._parse_table(schema=True),
             where=self._parse_where(),
         )
 
@@ -558,7 +605,7 @@ class Parser:
         return self.expression(
             exp.Update,
             **{
-                "this": self._parse_table(alias=None, schema=True),
+                "this": self._parse_table(schema=True),
                 "expressions": self._match(TokenType.SET)
                 and self._parse_csv(self._parse_equality),
                 "from": self._parse_from(),
@@ -578,7 +625,7 @@ class Parser:
     def _parse_cache(self):
         lazy = self._match(TokenType.LAZY)
         self._match(TokenType.TABLE)
-        table = self._parse_table(alias=None, schema=True)
+        table = self._parse_table(schema=True)
         options = []
 
         if self._match(TokenType.OPTIONS):
@@ -595,7 +642,7 @@ class Parser:
             this=table,
             lazy=lazy,
             options=options,
-            expression=self._parse_cte(),
+            expression=self._parse_with(),
         )
 
     def _parse_partition(self):
@@ -632,32 +679,69 @@ class Parser:
         self._match_r_paren()
         return self.expression(exp.Tuple, expressions=expressions)
 
-    def _parse_cte(self):
+    def _parse_with(self):
         if not self._match(TokenType.WITH):
             return self._parse_select()
 
+        recursive = self._match(TokenType.RECURSIVE)
+
         expressions = []
 
         while True:
-            recursive = self._match(TokenType.RECURSIVE)
-            alias = self._parse_function() or self._parse_id_var()
-
-            if not alias:
-                self.raise_error("Expected alias after WITH")
-
-            if not self._match(TokenType.ALIAS):
-                self.raise_error("Expected AS after WITH")
-
-            expressions.append(self._parse_table(alias=alias))
+            expressions.append(self._parse_cte())
 
             if not self._match(TokenType.COMMA):
                 break
 
+        this = self._parse_statement()
+
+        if "with" not in this.arg_types:
+            self.raise_error(f"{this.key} does not support CTE")
+
+        this.set(
+            "with",
+            self.expression(
+                exp.With,
+                expressions=expressions,
+                recursive=recursive,
+            ),
+        )
+        return this
+
+    def _parse_cte(self):
+        alias = self._parse_table_alias()
+        if not alias or not alias.this:
+            self.raise_error("Expected CTE to have alias")
+
+        if not self._match(TokenType.ALIAS):
+            self.raise_error("Expected AS in CTE")
+
+        self._match_l_paren()
+        expression = self._parse_select()
+        self._match_r_paren()
+
         return self.expression(
             exp.CTE,
-            this=self._parse_statement(),
-            expressions=expressions,
-            recursive=recursive,
+            this=expression,
+            alias=alias,
+        )
+
+    def _parse_table_alias(self):
+        self._match(TokenType.ALIAS)
+        alias = self._parse_id_var()
+        columns = None
+
+        if self._match(TokenType.L_PAREN):
+            columns = self._parse_csv(self._parse_id_var)
+            self._match_r_paren()
+
+        if not alias and not columns:
+            return None
+
+        return self.expression(
+            exp.TableAlias,
+            this=alias,
+            columns=columns,
         )
 
     def _parse_select(self):
@@ -666,9 +750,13 @@ class Parser:
         if self._match(TokenType.L_PAREN):
             this = self._parse_select()
             self._match_r_paren()
-            this = self._parse_alias(this)
+            this = self.expression(
+                exp.Subquery,
+                this=this,
+                alias=self._parse_table_alias(),
+            )
 
-        if isinstance(this, exp.Alias) or self._match(TokenType.SELECT):
+        if isinstance(this, exp.Subquery) or self._match(TokenType.SELECT):
             this = self.expression(
                 exp.Select,
                 hint=self._parse_hint(),
@@ -714,68 +802,58 @@ class Parser:
         return self.expression(exp.From, expressions=self._parse_csv(self._parse_table))
 
     def _parse_laterals(self):
-        laterals = []
+        return self._parse_all(self._parse_lateral)
 
-        while True:
-            if not self._match(TokenType.LATERAL):
-                return laterals
+    def _parse_lateral(self):
+        if not self._match(TokenType.LATERAL):
+            return None
 
-            if not self._match(TokenType.VIEW):
-                self.raise_error("Expected VIEW afteral LATERAL")
+        if not self._match(TokenType.VIEW):
+            self.raise_error("Expected VIEW after LATERAL")
 
-            outer = self._match(TokenType.OUTER)
-            this = self._parse_function()
-            table = self._parse_id_var()
-            columns = (
-                self._parse_csv(self._parse_id_var)
-                if self._match(TokenType.ALIAS)
-                else None
-            )
+        outer = self._match(TokenType.OUTER)
+        this = self._parse_function()
+        table = self._parse_id_var()
+        columns = (
+            self._parse_csv(self._parse_id_var)
+            if self._match(TokenType.ALIAS)
+            else None
+        )
 
-            laterals.append(
-                self.expression(
-                    exp.Lateral,
-                    this=this,
-                    outer=outer,
-                    table=self.expression(exp.Table, this=table),
-                    columns=columns,
-                )
-            )
+        return self.expression(
+            exp.Lateral,
+            this=this,
+            outer=outer,
+            table=self.expression(exp.Table, this=table),
+            columns=columns,
+        )
 
     def _parse_joins(self):
-        joins = []
+        return self._parse_all(self._parse_join)
 
-        while True:
-            side = (
-                self._match_set((TokenType.LEFT, TokenType.RIGHT, TokenType.FULL))
-                and self._prev
-            )
-            kind = (
-                self._match_set((TokenType.INNER, TokenType.OUTER, TokenType.CROSS))
-                and self._prev
-            )
+    def _parse_join(self):
+        side = self._match_set(self.JOIN_SIDES) and self._prev
+        kind = self._match_set(self.JOIN_KINDS) and self._prev
 
-            if not self._match(TokenType.JOIN):
-                return joins
+        if not self._match(TokenType.JOIN):
+            return None
 
-            joins.append(
-                self.expression(
-                    exp.Join,
-                    this=self._parse_table(),
-                    side=side.text if side else None,
-                    kind=kind.text if kind else None,
-                    on=self._parse_conjunction() if self._match(TokenType.ON) else None,
-                )
-            )
+        return self.expression(
+            exp.Join,
+            this=self._parse_table(),
+            side=side.text if side else None,
+            kind=kind.text if kind else None,
+            on=self._parse_conjunction() if self._match(TokenType.ON) else None,
+        )
 
-    def _parse_table(self, alias=False, schema=False):
+    def _parse_table(self, schema=False):
         unnest = self._parse_unnest()
 
         if unnest:
             return unnest
 
         if self._match(TokenType.L_PAREN):
-            expression = self._parse_cte()
+            this = self._parse_with()
             self._match_r_paren()
         else:
             catalog = None
@@ -789,22 +867,18 @@ class Parser:
                 if not table:
                     self.raise_error("Expected table name")
 
-            expression = self.expression(exp.Table, this=table, db=db, catalog=catalog)
-
-        expression = self._parse_table_sample(expression)
-
-        if alias is None:
-            this = expression
-        elif alias:
-            this = self.expression(exp.Alias, this=expression, alias=alias)
-        else:
-            this = self._parse_alias(expression)
-
-        if not isinstance(this, (exp.Alias, exp.Table)):
-            this = self.expression(exp.Alias, this=this, alias=None)
+            this = self.expression(exp.Table, this=table, db=db, catalog=catalog)
 
         if schema:
-            return self._parse_schema(this=expression)
+            return self._parse_schema(this=this)
+
+        this = self._parse_table_sample(this)
+        alias = self._parse_table_alias()
+
+        if isinstance(this, self.BODY_EXPRESSIONS):
+            return self.expression(exp.Subquery, this=this, alias=alias)
+        if alias:
+            return self.expression(exp.Alias, this=this, alias=alias)
         return this
 
     def _parse_unnest(self):
@@ -1452,6 +1526,9 @@ class Parser:
 
         return this
 
+    def _parse_all(self, parse):
+        return list(iter(parse, None))
+
     def _match(self, token_type):
         if not self._curr:
             return None
